/* Copyright (C) 1995,1996,1997,1998,2001,2002 Free Software Foundation, Inc.
   This file is part of the GNU C Library.
   Contributed by Ulrich Drepper <drepper@gnu.ai.mit.edu>, August 1995.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#include <stdlib.h>
#include <stdio.h>
double seed;		/* assumed initialized with a seed > 0 */
#define A 16807.0
#define M 2147483647.0
#define Q 127773.0	/* M div A */
#define R 2836.0	/* M mod A */

/*
 * Initialize the PRNG with a seed > 0.
 */
void ldpc_srand (int s)
{
	if (s > 0)
	{
		seed = s;
	}
	else		
		exit(-1);
}



/* Returns a random integer in [0; maxv-1]
 * derived from [Park88].
 */
int ldpc_rand (int	maxv)
{
	double lo, hi, test;
	double rand;

	hi = (int) (seed / Q);
	lo = seed - Q*hi;
	test = A*lo - R*hi;
	if (test > 0.0)
		seed = test;
	else
		seed = test + M;
	
	rand = seed / M;
	
	if (rand == 1.0)
		return 0;
	else
		return ((int)(rand * (double)maxv));
}
